
<!-- Format Toggle Buttons -->
<div class="btn-group btn-group-sm" role="group" aria-label="Format toggle buttons">
    <button id="toggle-yaml-btn" type="button" class="btn btn-outline-secondary">
        <i class="mdi mdi-code-braces"></i>
        <span>Switch to YAML</span>
    </button>
    <button id="toggle-json-btn" type="button" class="btn btn-outline-secondary">
        <i class="mdi mdi-code-json"></i>
        <span>Switch to JSON</span>
    </button>
</div>
