
<!-- Theme Selector Dropdown -->
<div class="dropdown">
    <button class="btn btn-sm btn-outline-secondary dropdown-toggle"
            type="button"
            id="theme-dropdown"
            data-bs-toggle="dropdown"
            aria-expanded="false">
        <i class="mdi mdi-palette"></i>
        <span>Light</span>
    </button>
    <ul class="dropdown-menu" aria-labelledby="theme-dropdown">
        <li><a class="dropdown-item" href="#" data-theme="vs">Light</a></li>
        <li><a class="dropdown-item" href="#" data-theme="vs-dark">Dark</a></li>
        <li><a class="dropdown-item" href="#" data-theme="hc-black">High Contrast Dark</a></li>
        <li><a class="dropdown-item" href="#" data-theme="hc-light">High Contrast Light</a></li>
    </ul>
</div>
