# Generated by Django 5.1.8 on 2025-10-17 11:40

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("netbox_panorama_configpump_plugin", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="deviceconfigsyncstatus",
            name="deduce_xpaths",
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name="deviceconfigsyncstatus",
            name="manual_xpath_entries",
            field=django.contrib.postgres.fields.ArrayField(
                base_field=models.CharField(max_length=1024),
                blank=True,
                default=list,
                size=None,
            ),
        ),
    ]
