# Generated by Django 5.1.8 on 2026-03-03 13:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "netbox_panorama_configpump_plugin",
            "0002_deviceconfigsyncstatus_deduce_xpaths_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="connectiontemplate",
            name="file_name_prefix",
            field=models.CharField(blank=True, max_length=16),
        ),
    ]
